'
' Example DLL to display a modal dialog and return a value to the calling
' application.

#COMPILE DLL

#INCLUDE "WIN32API.INC"
#RESOURCE "MyInputBox.pbr"

GLOBAL hCurInstance AS DWORD

GLOBAL gText        AS ASCIIZ * 255
GLOBAL gCaption     AS ASCIIZ * 255
GLOBAL gDefault     AS ASCIIZ * 255

'-------------------------------------------------------------------------------
' Main DLL entry point called by Windows...
'
FUNCTION LIBMAIN (BYVAL hInstance   AS LONG, _
                  BYVAL fwdReason   AS LONG, _
                  BYVAL lpvReserved AS LONG) AS LONG

   SELECT CASE fwdReason

   CASE %DLL_PROCESS_ATTACH
      'Indicates that the DLL is being loaded by another process (a DLL
      'or EXE is loading the DLL).  DLLs can use this opportunity to
      'initialize any instance or global data, such as arrays.

      hCurInstance = hInstance

      FUNCTION = 1   'success!

      'FUNCTION = 0   'failure!  This will prevent the EXE from running.

   CASE %DLL_PROCESS_DETACH
      'Indicates that the DLL is being unloaded or detached from the
      'calling application.  DLLs can take this opportunity to clean
      'up all resources for all threads attached and known to the DLL.

      FUNCTION = 1   'success!

      'FUNCTION = 0   'failure!

   CASE %DLL_THREAD_ATTACH
      'Indicates that the DLL is being loaded by a new thread in the
      'calling application.  DLLs can use this opportunity to
      'initialize any thread local storage (TLS).

      FUNCTION = 1   'success!

      'FUNCTION = 0   'failure!

   CASE %DLL_THREAD_DETACH
      'Indicates that the thread is exiting cleanly.  If the DLL has
      'allocated any thread local storage, it should be released.

      FUNCTION = 1   'success!

      'FUNCTION = 0   'failure!

   END SELECT

END FUNCTION


FUNCTION MyInputBox ALIAS "MyInputBox" ( _
                    BYVAL strText AS STRING, _
                    BYVAL strCaption AS STRING, _
                    BYREF szDefault AS ASCIIZ) EXPORT AS LONG

   ' If pointers are non-null, set default values for dialog
   IF strText <> "" THEN
      gText = strText
   ELSE
      gText = "Enter Text"
   END IF

   IF strCaption <> "" THEN
      gCaption = strCaption
   ELSE
      gCaption = "INPUT BOX"
    END IF

   IF VARPTR(szDefault) THEN
      gDefault = szDefault
   ELSE
      gDefault = ""
   END IF

   ' Pop up the modal dialog and get return value
   FUNCTION = DialogBoxParam(hCurInstance, "INPUTBOX", 0, CODEPTR(InputDlgProc), 0)

   ' If default buffer was passed, return the input text
   IF VARPTR(szDefault) THEN
      szDefault = gDefault
   END IF

END FUNCTION


' **
' ** Generic routine to center any window given its handle.
' **
SUB CenterWindow(BYVAL hWnd AS WORD)

   DIM WndRect AS RECT
   DIM x       AS INTEGER
   DIM y       AS INTEGER

   GetWindowRect hWnd, WndRect

   x = (GetSystemMetrics(%SM_CXSCREEN)-(WndRect.nRight-WndRect.nLeft))\2
   y = (GetSystemMetrics(%SM_CYSCREEN)-(WndRect.nBottom-WndRect.nTop+GetSystemMetrics(%SM_CYCAPTION)))\2

   SetWindowPos hWnd, %NULL, x, y, 0, 0, %SWP_NOSIZE OR %SWP_NOZORDER

END SUB


FUNCTION InputDlgProc(BYVAL hDlg AS INTEGER, _
                      BYVAL wMsg AS INTEGER, _
                      BYVAL wParam AS INTEGER, _
                      BYVAL lParam AS LONG) EXPORT AS LONG

   ' See INPUTBOX.RC script for details on the dialog itself

   SELECT CASE wMsg
      CASE %WM_INITDIALOG

         'Center the window
         CenterWindow hDlg

         'Assign the initial values to the dialog
         SetWindowText GetDlgItem(hDlg, 101), gText
         SetWindowText GetDlgItem(hDlg, 102), gDefault
         SetWindowText hDlg, gCaption

         FUNCTION = 1
         EXIT FUNCTION

      CASE %WM_COMMAND
         SELECT CASE wParam
            CASE 103, %IDOK
               'When the "OK" button is pressed, get the edit box value
               ' and end the dialog
               GetWindowText GetDlgItem(hDlg, 102), gDefault, 255
               EndDialog hDlg, 1
               FUNCTION = 1
               EXIT FUNCTION
               'When the "CANCEL" button is pressed, or escape is pressed, or
               ' the dialog is closed, end the dialog
            CASE 104, %IDCANCEL
               EndDialog hDlg, 0
               FUNCTION = 1
               EXIT FUNCTION
         END SELECT
   END SELECT

END FUNCTION
